package com.doc.chat.controller;

import java.util.Date;
import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson2.JSON;
import com.doc.biz.domain.DocMsg;
import com.doc.common.constant.CacheConstants;
import com.doc.common.constant.Constants;
import com.doc.common.utils.SecurityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.doc.common.annotation.Log;
import com.doc.common.core.controller.BaseController;
import com.doc.common.core.domain.AjaxResult;
import com.doc.common.enums.BusinessType;
import com.doc.chat.domain.ChatMsg;
import com.doc.chat.service.IChatMsgService;
import com.doc.common.utils.poi.ExcelUtil;
import com.doc.common.core.page.TableDataInfo;

/**
 * 聊天消息管理Controller
 *
 * @author wukai
 * @date 2023-08-24
 */
@Api(tags = "聊天消息管理")
@RestController
@RequestMapping("/chat/msg")
public class ChatMsgController extends BaseController {
    @Resource
    private IChatMsgService chatMsgService;
    @Resource
    private StringRedisTemplate stringRedisTemplate;

    /**
     * 发送消息
     *
     * @param msg
     * @return
     */
    @ApiOperation("发送消息")
    @PostMapping("/send")
    public AjaxResult send(@RequestBody ChatMsg msg) {
        msg.setFromId(SecurityUtils.getUserId());
        msg.setCreateTime(new Date());
        stringRedisTemplate.convertAndSend(CacheConstants.CHANNEL, JSON.toJSONString(msg));
        chatMsgService.insertChatMsg(msg);
        return success();
    }

    /**
     * 消息记录
     *
     * @param toId 用户ID
     * @return
     */
    @ApiOperation("消息记录")
    @GetMapping("/record/{toId}")
    public TableDataInfo record(@PathVariable Long toId) {
        startPage();
        List<ChatMsg> list = chatMsgService.selectRecordList(SecurityUtils.getUserId(), toId);
        return getDataTable(list);
    }

    /**
     * 好友列表功能
     */
    @ApiOperation("好友列表")
    @GetMapping("/friend")
    public TableDataInfo friend() {
        List<ChatMsg> list = chatMsgService.selectFriendList(SecurityUtils.getUserId());
        return getDataTable(list);
    }

    /**
     * 查询聊天消息管理列表
     */
    @ApiOperation("查询聊天消息管理列表")
    @PreAuthorize("@ss.hasPermi('chat:msg:list')")
    @GetMapping("/list")
    public TableDataInfo list(ChatMsg chatMsg) {
        startPage();
        List<ChatMsg> list = chatMsgService.selectChatMsgList(chatMsg);
        return getDataTable(list);
    }

    /**
     * 导出聊天消息管理列表
     */
    @ApiOperation("导出聊天消息管理列表")
    @PreAuthorize("@ss.hasPermi('chat:msg:export')")
    @Log(title = "聊天消息管理" , businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ChatMsg chatMsg) {
        List<ChatMsg> list = chatMsgService.selectChatMsgList(chatMsg);
        ExcelUtil<ChatMsg> util = new ExcelUtil<ChatMsg>(ChatMsg.class);
        util.exportExcel(response, list, "聊天消息管理数据");
    }

    /**
     * 获取聊天消息管理详细信息
     */
    @ApiOperation("获取聊天消息管理详细信息")
    @PreAuthorize("@ss.hasPermi('chat:msg:query')")
    @GetMapping(value = "/{msgId}")
    public AjaxResult getInfo(@PathVariable("msgId") Long msgId) {
        return success(chatMsgService.selectChatMsgByMsgId(msgId));
    }

    /**
     * 新增聊天消息管理
     */
    @ApiOperation("新增聊天消息管理")
    @PreAuthorize("@ss.hasPermi('chat:msg:add')")
    @Log(title = "聊天消息管理" , businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody ChatMsg chatMsg) {
        return toAjax(chatMsgService.insertChatMsg(chatMsg));
    }

    /**
     * 修改聊天消息管理
     */
    @ApiOperation("修改聊天消息管理")
    @PreAuthorize("@ss.hasPermi('chat:msg:edit')")
    @Log(title = "聊天消息管理" , businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody ChatMsg chatMsg) {
        return toAjax(chatMsgService.updateChatMsg(chatMsg));
    }

    /**
     * 删除聊天消息管理
     */
    @ApiOperation("删除聊天消息管理")
    @PreAuthorize("@ss.hasPermi('chat:msg:remove')")
    @Log(title = "聊天消息管理" , businessType = BusinessType.DELETE)
    @DeleteMapping("/{msgIds}")
    public AjaxResult remove(@PathVariable Long[] msgIds) {
        return toAjax(chatMsgService.deleteChatMsgByMsgIds(msgIds));
    }
}
